package org.basex.examples.xmldb;

import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.basex.api.xmldb.BXCollection;

import org.w3c.dom.Document;

import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Database;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;


/**
 * This class serves as an example for creating a database with the XML:DB API.
 *
 * @author BaseX Team 2005-11, BSD License
 */
public final class Test {
   /** Database driver. */
   public static final String DRIVER = "org.basex.api.xmldb.BXDatabase";

   /**
    * Main method of the example class.
    * @param args (ignored) command-line arguments
    * @throws Exception exception
    */
   public static void main(final String[] args) throws Exception {

      System.out.println("=== XMLDBCreate ===");

      try {
         // Register the database
         Class<?> c = Class.forName(DRIVER);
         Database db = (Database)c.newInstance();
         DatabaseManager.registerDatabase(db);

         System.out.println("\n* Create a new collection.");

         // Create a new collection
         BXCollection col = new BXCollection("Collection", false);

         // Content of the first document
         String stringDoc1 =
            "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" + "<tag xmlns=\"http://example.org\">" +
            "</tag>";
         // Content of the second document
         String stringDoc2 =
            "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" + "<tns:tag xmlns:tns=\"http://example.org\">" +
            "</tns:tag>";

         // ID for the first document
         String id1 = "string-doc1";
         // ID for the second document
         String id2 = "string-doc2";
         // ID for the tird document
         String id3 = "dom-doc1";
         // ID for the fourth document
         String id4 = "dim-doc2";
         // ID for the fifth document
         String id5 = "sax-doc1";
         // ID for the sixth document
         String id6 = "sax-doc2";

         DocumentBuilderFactory factoru = DocumentBuilderFactory.newInstance();
         DocumentBuilder builder = factoru.newDocumentBuilder();
         Document domDoc1 =
            builder.parse(new InputSource(new StringReader(stringDoc1)));
         Document domDoc2 =
            builder.parse(new InputSource(new StringReader(stringDoc2)));

         XMLReader reader = XMLReaderFactory.createXMLReader();

         System.out.println("\n* Create new resources.");

         // Create a new XML resource with the specified ID
         XMLResource res1 = col.createResource(id1, XMLResource.RESOURCE_TYPE);
         XMLResource res2 = col.createResource(id2, XMLResource.RESOURCE_TYPE);
         XMLResource res3 = col.createResource(id3, XMLResource.RESOURCE_TYPE);
         XMLResource res4 = col.createResource(id4, XMLResource.RESOURCE_TYPE);
         XMLResource res5 = col.createResource(id5, XMLResource.RESOURCE_TYPE);
         XMLResource res6 = col.createResource(id6, XMLResource.RESOURCE_TYPE);

         // Set the content of the XML resource as the document
         res1.setContent(stringDoc1);
         res2.setContent(stringDoc2);
         res3.setContentAsDOM(domDoc1);
         res4.setContentAsDOM(domDoc2);
         ContentHandler handler = res5.setContentAsSAX();
         reader.setContentHandler(handler);
         reader.parse(new InputSource(new StringReader(stringDoc1)));
         handler = res6.setContentAsSAX();
         reader.setContentHandler(handler);
         reader.parse(new InputSource(new StringReader(stringDoc2)));


         System.out.println("\n* Store new resource.");

         // Store the resource into the database
         col.storeResource(res1);
         col.storeResource(res2);
         col.storeResource(res3);
         col.storeResource(res4);
         col.storeResource(res5);
         col.storeResource(res6);

         col.close();

      } catch (final XMLDBException ex) {
         // Handle exceptions
         System.err.println("XML:DB Exception occured " + ex.errorCode);
      }
   }
}
